import time
from io import BytesIO

from PIL import Image
from bs4 import BeautifulSoup

from ..AuthSession import AuthSession
from ..utils.page import parse_form_hidden_inputs
from ..utils.vcode import _process_vcode
from ..utils.aes import encrypt


class IDSSession(AuthSession):
    cookie_name = 'ids'

    def __init__(
            self, target, username, password,
            *args, **kwargs
    ):
        super().__init__(f'{self.cookie_name}_{username}')
        if self.is_logged_in():
            return
        else:
            self.cookies.clear()
        page = self.get(
            'http://ids.xidian.edu.cn/authserver/login',
            params={'service': target, 'type': 'userNameLogin'}
        ).text
        is_need_captcha, vcode = self.get(
            'https://ids.xidian.edu.cn/authserver/checkNeedCaptcha.htl',
            params={'username': username, '_': int(time.time() * 1000)}
        ).json()['isNeed'], None
        if is_need_captcha:
            captcha = self.get(
                'http://ids.xidian.edu.cn/authserver/getCaptcha.htl',
                params={str(int(time.time() * 1000)): ''}
            )
            _process_vcode(Image.open(BytesIO(captcha.content))).show()
            vcode = input('验证码：')
        page = BeautifulSoup(page, "lxml")
        form = page.findChild(attrs={'id': 'pwdFromId'})
        params = parse_form_hidden_inputs(form)
        enc = form.find('input', id='pwdEncryptSalt').get('value')
        self.post(
            'http://ids.xidian.edu.cn/authserver/login',
            params={'service': target},
            data=dict(params, **{
                'username': username,
                'password': encrypt(password.encode(), enc.encode()),
                'captcha': vcode,
                'rememberMe': 'true'
            })
        )

    def is_logged_in(self):
        return self.get(
            'http://ids.xidian.edu.cn/authserver/index.do',
            allow_redirects=False
        ).status_code != 302



from io import BytesIO

from PIL import Image
from bs4 import BeautifulSoup

from ..AuthSession import AuthSession
from ..utils.page import parse_form_hidden_inputs
from ..utils.vcode import _process_vcode


class RSBBSSession(AuthSession):
    HOST = 'rsbbs.xidian.edu.cn'
    cookie_name = 'rsbbs'

    def __init__(self, username, password):
        super().__init__(f'{self.cookie_name}_{username}')
        if not self.is_loggedin():
            self.login(username, password)

    def login(self, username, password):
        login = self.get(f'http://{self.HOST}/member.php', params={
            'mod': 'logging',
            'action': 'login',
            'mobile': '2'
        })
        soup = BeautifulSoup(login.text, 'lxml')

        img = soup.find('img', {'class': 'seccodeimg'}).get('src')
        img = _process_vcode(Image.open(
            BytesIO(self.get(f'http://{self.HOST}/{img}', headers={
                'Referer': login.url
            }).content)
        ))
        img.show()
        vcode = input('验证码：')
        page = self.post(
            f'http://{self.HOST}/' +
            soup.find('form', id='loginform').get('action'), data=dict(
                parse_form_hidden_inputs(soup), **{
                    'username': username,
                    'password': password,
                    'questionid': '0',
                    'answer': '',
                    'seccodeverify': vcode,
                }
            )
        ).text
        if '欢迎您回来' not in page:
            return self.login(username, password)
        return

    def is_loggedin(self):
        return self.get(f'http://{self.HOST}/home.php', params={
            'mod': 'space', 'do': 'profile'
        }, allow_redirects=False).status_code != 302



from .ids import IDSSession
from ..AuthSession import AuthSession


class EhallSession(IDSSession):
    cookie_name = 'ehall'

    def __init__(self, username, password, *args, **kwargs):
        AuthSession.__init__(self, f'{self.cookie_name}_{username}')
        if not self.is_logged_in():
            super().__init__(
                'http://ehall.xidian.edu.cn/login?service=http://ehall.xidian.edu.cn/new/index.html',
                username, password, *args, **kwargs
            )

    def use_app(self, app_id):
        self.get('http://ehall.xidian.edu.cn//appShow', params={
            'appId': app_id
        }, headers={
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8",
        })

    def get_app_list(self, search_key=''):
        app_list = self.get('http://ehall.xidian.edu.cn/jsonp/serviceSearchCustom.json', params={
            'searchKey': search_key,
            'pageNumber': 1,
            'pageSize': 150,
            'sortKey': 'recentUseCount',
            'orderKey': 'desc'
        }).json()
        assert app_list['hasLogin']
        return app_list['data']

    def get_app_id(self, search_key):
        search_result = self.get_app_list(search_key)
        if len(search_result) == 0:
            return None
        if len(search_result) > 1:
            # warn('multiple results found, returning the first one')
            pass
        return search_result[0]['appId']

    def is_logged_in(self):
        return self.get('http://ehall.xidian.edu.cn/jsonp/userFavoriteApps.json').json()['hasLogin']



class Processor:
    def __init__(self, img):
        self.img = img.convert('L')
        self.img_arr = self.img.load()
        self.paint()

    DX = [1, 0, -1, 0]
    DY = [0, 1, 0, -1]

    def paint(self):
        w, h = self.img.size
        visited = set()
        q = []
        q.append((0, 0, 255))
        while q:
            x, y, value = q.pop()
            if x < 0 or y < 0 or x >= w or y >= h or \
                    (x, y) in visited:
                continue
            visited.add((x, y))
            for i in range(4):
                try:
                    pixel = self.img_arr[x + self.DX[i], y + self.DY[i]]
                except IndexError:
                    continue
                if abs(pixel - self.img_arr[x, y]) > 5:
                    q.append((x + self.DX[i], y + self.DY[i], 255 - value))
                else:
                    q.append((x + self.DX[i], y + self.DY[i], value))
            self.img_arr[x, y] = value


def _process_vcode(img):
    p = Processor(img)
    return p.img



import hashlib
import json
import random
import time

from requests import Session


def _generate_uuid():
    a = [str(random.random())[2:10] for i in range(2)]
    a = [a[i] + str(int(time.time() * 1000))[-10:] for i in range(2)]
    a = [hex(int(a[i]))[2:10] for i in range(2)]
    return "web" + a[0] + a[1]


class WXSession(Session):
    BASE = 'http://202.117.121.7:8080/'

    def _dump_sign(self, data):
        l = list(data.keys())
        l.sort()
        s = ''
        for i in l:
            s += i + '=' + str(data[i]) + '&'
        s = s[:-1]
        return hashlib.md5(s.encode('utf-8')).hexdigest()

    def options(self, url):
        return super().options(url, headers={
            'Access-Control-Request-Headers': 'content-type,token',
            'Access-Control-Request-Method': 'POST'
        })

    def post(self, url, data=None, json=None, headers=None, param=None):
        self.options(url)
        if param is not None:
            json = {
                'appKey': "GiITvn",
                'param': param,
                'secure': 0
            }
        if json is not None:
            json['time'] = int(time.time() * 1000)  # 先后顺序
            json['sign'] = self._dump_sign(json)  # 数据签名在生成时间戳之后
            if headers == None:
                headers = {}
            headers = dict(headers, **{
                'Content-Type': 'application/json;charset=UTF-8'
            })
        return super().post(url, json=json, data=data, headers=headers)

    def __init__(self, username, password, *args, **kwargs):
        super(WXSession, self).__init__(*args, **kwargs)
        data = {
            'appKey': "GiITvn",
            'param': json.dumps({
                'userName': username,
                'password': password,
                'schoolId': 190,
                'uuId': _generate_uuid()
            }),
            'secure': 0
        }
        result = self.post(self.BASE + 'baseCampus/login/login.do', json=data).json()
        if result['isConfirm'] != 1:
            raise ConnectionError('登录失败')  # 请检查credentials.py
        self.headers.update({
            'token': result['token'][0] + '_' + result['token'][1]
        })



import http.cookiejar
import os

from requests import Session


# noinspection PyTypeChecker
class AuthSession(Session):
    def __init__(self, name=None, cookies={}, headers={}):
        cookie_name = name if name else self.cookie_name
        self.cookie_path = os.path.expanduser(
            os.path.join('~', '.xduauth', 'cookies', cookie_name)
        )
        os.makedirs(os.path.dirname(self.cookie_path), exist_ok=True)
        if not cookies and hasattr(self, 'cookies'):
            cookies = self.cookies
        super().__init__()
        self.cookies = http.cookiejar.LWPCookieJar(self.cookie_path)

        for cookie in cookies:
            self.cookies.set_cookie(cookie)
        self.headers.update(headers)

        if not os.path.exists(self.cookie_path):
            self.cookies.save(ignore_discard=True)
        self.cookies.load(ignore_discard=True)

    def request(self, *args, **kwargs):
        ret = super().request(*args, **kwargs)
        self.cookies.save(ignore_discard=True)
        return ret



import json
from base64 import b64encode, b64decode
from io import BytesIO
from re import search

import requests
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad
from PIL import Image

from ..AuthSession import AuthSession


def encrypt(password, key):
    if type(password) is str:
        password = password.encode('utf-8')
    if type(key) is str:
        key = key.encode('utf-8')
    crypt = AES.new(key, AES.MODE_ECB)
    return b64encode(crypt.encrypt(pad(password, 16))).decode('utf-8')


class XKSession(AuthSession):
    BASE = 'https://xk.xidian.edu.cn/xsxk'
    cookie_name = 'xk'
    user = {}
    current_batch = {}

    def __init__(self, username, password):
        super().__init__(f'{self.cookie_name}_{username}')
        self.username = username
        cookies = requests.utils.dict_from_cookiejar(self.cookies)
        if 'token' in cookies and 'batch' in cookies:
            self.headers.update({'Authorization': cookies['token']})
            self.current_batch = json.loads(cookies['batch'])
        if 'token' not in cookies or 'batch' not in cookies or not self.is_loggedin():
            self.persist('token', self.login(username, password))
            self.current_batch = next(filter(
                lambda batch: batch['canSelect'] == '1',
                self.user['electiveBatchList']
            ))
            self.persist('batch', json.dumps(self.current_batch))
            cookies = requests.utils.dict_from_cookiejar(self.cookies)
            self.headers.update({'Authorization': cookies['token']})

    def persist(self, name, value):
        self.cookies.set_cookie(requests.cookies.create_cookie(
            domain='xk.xidian.edu.cn', path='/nowhere',
            name=name, value=value,
        ))
        self.cookies.save(ignore_discard=True)

    def login(self, username, password):
        self.headers.update({'Authorization': ''})
        index = self.get(self.BASE + '/xsxk/profile/index.html').text
        key = search(r'loginForm\.aesKey = "([a-zA-Z0-9=]*)";', index)
        if not key:  # 应当不会出现 index.html 中没有 aeskey 的情况
            raise RuntimeError('AES Key Not Found')
        key = key.groups()[0]

        resp = self.post(self.BASE + '/auth/captcha')
        captcha = resp.json()['data']

        # show captcha
        captcha_img = captcha['captcha'][22:]  # data:image/png;base64,
        Image.open(BytesIO(b64decode(captcha_img))).show()

        # TODO:input function shouldn't exist here
        captcha_code = input('验证码：')

        login_resp = self.post(f'{self.BASE}/auth/login', data={
            'loginname': username,
            'password': encrypt(password, key),
            'captcha': captcha_code,
            'uuid': captcha['uuid']
        }).json()
        if login_resp['code'] != 200:
            raise RuntimeError(login_resp['msg'])
        self.user = login_resp['data']['student']
        return login_resp['data']['token']

    def is_loggedin(self):
        try:
            info = self.post(f'{self.BASE}/elective/user', data={
                'batchId': self.current_batch['code'],
            }).json()
        except requests.JSONDecodeError:
            return False
        if info['code'] == 200:
            self.user = info['data']['student']
            return True
        return False



from requests import Session


class EnergySession(Session):
    BASE = 'http://10.168.55.50:8088'

    def __init__(self, username, password, *args, **kwargs):
        super(EnergySession, self).__init__(*args, **kwargs)
        self.get(self.BASE + "/searchWap/Login.aspx")
        self.post(
            self.BASE + "/ajaxpro/SearchWap_Login,App_Web_fghipt60.ashx",
            json={
                "webName": username,
                "webPass": password
            }, headers={
                "AjaxPro-Method": "getLoginInput",
                'Origin': self.BASE
            }
        )



import warnings

from requests import get

package = __package__.split('.')[0]


def get_self_version(package):
    try:
        from importlib.metadata import version
        return version(package)
    except:
        from pkg_resources import get_distribution
        return get_distribution(package)
    warnings.warn('unable to get local package version')


def get_latest_version(package, source='pypi.org'):
    pkginfo = get(f'https://{source}/pypi/{package}/json').json()
    return pkginfo['info']['version']


try:
    latest = get_latest_version(package)
    if get_self_version(package) != latest:
        warnings.warn(f'''
You are not using the latest version of libxduauth({latest}).
upgrade the package by running:
`pip3 install --upgrade libxduauth`
''')
except:
    pass



from Crypto.Cipher import AES
from Crypto.Util.Padding import pad
from base64 import b64encode


def encrypt(v, k):
    crypt = AES.new(k, AES.MODE_CBC, b'xidianscriptsxdu')
    return b64encode(crypt.encrypt(pad(b'xidianscriptsxdu' * 4 + v, 16)))



from .sites.ehall import EhallSession
from .sites.energy import EnergySession
from .sites.ids import IDSSession
from .sites.rsbbs import RSBBSSession
from .sites.wx import WXSession
from .sites.xk import XKSession
from .sites.zfw import ZFWSession
from .sites.sports import SportsSession

import libxduauth.utils.update



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="libxduauth",
    version="1.7.2",
    author="Frank",
    author_email="frankli0324@hotmail.com",
    description="login utilities for XDU",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/xdlinux/libxdauth",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.7',
    install_requires=[
        "requests", "bs4", "pycryptodome",
        "importlib-resources", "Pillow",
    ],
)



import base64
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5 as PKCS1_cipher

def rsa_encrypt_by_pkcs1(public_key, data):
    cipher = PKCS1_cipher.new(RSA.importKey(public_key))
    rsa_text = base64.b64encode(cipher.encrypt(bytes(data.encode("utf8"))))
    return rsa_text.decode('utf-8')



import hashlib
from datetime import datetime
from requests import Session

from ..utils.rsa import rsa_encrypt_by_pkcs1


class SportsSession(Session):
    BASE_URL = 'https://xd.boxkj.com/app/'

    __RSA_PUBLIC_KEY = """-----BEGIN RSA PUBLIC KEY-----
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAq4laolA7zAk7jzsqDb3Oa5pS/uCPlZfASK8Soh/NzEmry77QDZ2koyr96M5Wx+A9cxwewQMHzi8RoOfb3UcQO4UDQlMUImLuzUnfbk3TTppijSLH+PU88XQxcgYm2JTa546c7JdZSI6dBeXOJH20quuxWyzgLk9jAlt3ytYygPQ7C6o6ZSmjcMgE3xgLaHGvixEVpOjL/pdVLzXhrMqWVAnB/snMjpCqesDVTDe5c6OOmj2q5J8n+tzIXtnvrkxQSDaUp8DWF8meMwyTErmYklMXzKic2rjdYZpHh4x98Fg0Q28sp6i2ZoWiGrJDKW29mntVQQiDNhKDawb4B45zUwIDAQAB
-----END RSA PUBLIC KEY-----"""

    __COMMON_HEADERS = {
        'channel': 'H5',
        'version': '99999',
        'type': '0'
    }
    __COMMON_SIGN_PARAMS = {
        'appId': '3685bc028aaf4e64ad6b5d2349d24ba8',
        'appSecret': 'e8167ef026cbc5e456ab837d9d6d9254'
    }

    user_id = ''

    def __get_sign(self, params):
        sorted_params = sorted(params.items())
        concated_params = '&'.join(
            ['{}={}'.format(entry[0], entry[1]) for entry in sorted_params])
        return hashlib.md5(concated_params.encode('utf-8')).hexdigest()

    def __append_payload_sign(self, payload):
        timestamp = int(datetime.now().timestamp() * 1000)
        payload['timestamp'] = timestamp
        sign_params = payload.copy()
        sign_params.update(self.__COMMON_SIGN_PARAMS)
        payload['sign'] = self.__get_sign(sign_params)

    def post(self, url, data):
        payload = data.copy()
        self.__append_payload_sign(payload)

        return super().post(url, data=payload, headers=self.__COMMON_HEADERS)

    def __init__(self, username, password, *args, **kwargs):
        super(SportsSession, self).__init__(*args, **kwargs)
        login_response = self.post(
            self.BASE_URL + 'h5/login', data={
                'uname': username,
                'pwd': rsa_encrypt_by_pkcs1(self.__RSA_PUBLIC_KEY, password)
            }).json()
        if login_response['returnCode'] != '200':
            raise ConnectionError('登录失败: ' + login_response['returnMsg'])

        self.user_id = login_response['data']['id']
        self.headers.update({
            'token': login_response['data']['token']
        })



import re

from bs4 import BeautifulSoup
from requests import Session


class ZFWSession(Session):
    BASE = 'https://zfw.xidian.edu.cn'

    def __init__(self, username, password, *args, **kwargs):
        super(ZFWSession, self).__init__(*args, **kwargs)
        self.headers.update({
            'User-Agent': 'Mobile'
        })
        soup = BeautifulSoup(self.get(self.BASE).text, "lxml")
        vcv = soup.find('input', type='hidden').get('value')
        error = re.findall(
            r'请修复以下错误<\/p><ul><li>(.*?)<',
            self.post(self.BASE + '/login', data={
                "LoginForm[username]": username,
                "LoginForm[password]": password,
                "_csrf": vcv,
                "login-button": ""
            }).text
        )
        self.headers.pop('User-Agent')
        if len(error) > 0:
            raise ConnectionError(error[0])
